#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

/* Arrays fr Haeufigkeiten der Ziffern 1 bis 9 */
int hsize[9]; 
int htime[9];
/* Anzahl gelesener Dateien und Verzeichnisse */
int files, dirs;

/* Verzeichnisinhalt lesen */
void leseverzeichnis(DIR *dp, char *verzeichnis) {

	struct stat inode;		/* Platz fuer Stat-Infos  */
	struct dirent *dname;	/* Resultat von readdir() */
	char buf[200];			/* Platz fuer den Pfad	  */
	DIR *mydp = NULL; 		/* Verzeichniszeiger      */
	int size, time;			/* Dateigroesse, Zeitst.  */

	/* Alle Verzeichniseintraege lesen */
	while ((dname = readdir(dp)) != NULL) {

		if (strcmp(dname -> d_name, ".") == 0 || 
			strcmp(dname -> d_name, "..") ==0 ) {
			continue;
		}

		/* Kompletten Pfad in buf abspeichern */

		/* Sonderfall fuer root... (Backslash entfernen) */
		if (strcmp(verzeichnis, "/\0") == 0) { 
			sprintf(buf, "/%s", dname->d_name);
		}
		else {
			sprintf(buf, "%s/%s", verzeichnis, dname->d_name);
		}
			
		/* Mit lstat() Informationen ueber Verzeichniseintraege holen. */
		/* Im Gegensatz zu stat liest lstat direkt die INode des */
		/* Symbolischen Links und nicht die INode der Datei, auf */
		/* die der symbolische Link verweist. Sonst knnen Zyklen */
		/* entstehen, die das Programm endlos laufen lassen. */
		if (lstat(buf, &inode) < 0) {
			fprintf(stderr, "CANNOT READ INFO OF %s\n", buf);
		}
		else {

			/* Dateityp feststellen */
			switch (inode.st_mode&S_IFMT) {

				/* Directory */
				case S_IFDIR:
					if (!(mydp = opendir(buf))) {	
						fprintf(stderr, "CANNOT OPEN DIR %s\n", buf);
					}
					else {
						dirs++;
						leseverzeichnis(mydp, buf);
						closedir(mydp);
					}
					break;

				 /* Regulaere Datei */
				case S_IFREG:
					size = inode.st_size;
					time = inode.st_mtime;
					while (size >= 10) {
						size = size / 10;
					}
					while (time >= 10) {
						time = time / 10;
					}

		  			printf("%20s | %17i (%i) | %17i (%i)\n", 
						dname -> d_name, inode.st_size, size, inode.st_mtime, time);

					hsize[size-1]++;
					htime[time-1]++;
					files++;
					break;
			}
		}		
	}
}

/* Haeufigkeiten ausgeben */
void ergebnis() {
	int i;
	for (i = 0; i < 9; i++) {
		printf("size[%i]: %7i   |   time[%i] %7i\n", i+1, hsize[i], i+1, htime[i]);
	}
	printf("Anzahl Dateien %i\n", files);
	printf("Anzahl Verzeichnisse %i\n", dirs);
}

int main(int argc, char **argv)
{	
	DIR *dp = NULL; /* Verzeichniszeiger  */

	/* Verzeichnis einlesen */
	if (argc != 2) {
		printf("usage: %s directory\n", argv[0]);
		return -1;
	}

	/* Angegebenes Verzeichnis oeffnen */
	if (!(dp = opendir(argv[1]))) {	
		fprintf(stderr, "CANNOT OPEN DIR %s\n", argv[1]);
		return -1;
	}

	leseverzeichnis(dp, argv[1]);

	closedir(dp);

	ergebnis();

	return 0;
}
